import bs
import bsUtils
import bsUI
import bsAnnounce
import bsSpaz
import bsMap
import random
import bsCoopGame
import bsBomb
import bsPowerup
import time

gDidInitialTransition = False

gStartTime = time.time()

class MainMenu(bs.Session):

    def __init__(self):

        global gDidInitialTransition

        random.seed(123)

        self._actors = []

        # init our parent class
        bs.Session.__init__(self)


        # self._shield = bs.NodeActor(None,bs.Actor.UNRETAINED,
        #                             'shield',{'color':(0.3,0.2,2.0),'radius':0.2,'translate':(0,6,0)},name='testShield')

        self.myName = bs.NodeActor(None,bs.Actor.UNRETAINED,
                                   'text',{'vAttach':'bottom',
                                           'hAlign':'center',
                                           'color':(0.5,0.6,0.5,1.0),
                                           'scale':[0.9] if bs.getInterfaceType() == 'phone' else [0.7],
                                           'translate':(0,10),
                                           'text':bs.getResource('copyrightText')},
                                   name='Copyright')

        if not gDidInitialTransition:
            bsUtils.animate(self.myName.node,'opacity',{2300:0,3000:1.0})

        self.version = bs.NodeActor(None,bs.Actor.UNRETAINED,
                                    'text',{'vAttach':'bottom',
                                            'hAttach':'right',
                                            'hAlign':'right',
                                            'color':(0.5,0.6,0.5,1.0),
                                            'scale':[0.9] if bs.getInterfaceType() == 'phone' else [0.7],
                                            'translate':(-10,0),
                                            'text':bs.getVersion()+(' ('+str(bs.getBuildNumber())+')' if bs.isTestBuild() else '')+(' '+'('+bs.getResource('debugText')+')' if bs.isDebugBuild() else '')},
                                        name='Version String')
        if not gDidInitialTransition:
            bsUtils.animate(self.version.node,'opacity',{2300:0,3000:1.0})

        # self.testLoc = bs.NodeActor(None,bs.Actor.UNRETAINED,
        #                              'locator',{'translate':(0,5,3),
        #                                         'shape':'circleOutline',
        #                                         'color':(1,1,1),
        #                                         'opacity':0.5,
        #                                         'drawBeauty':False,
        #                                         'size':[5]},
        #                              name='Test Locator')
        # bsUtils.animate(self.testLoc.node,'size',{0:1.0,2000:4.0,4000:1.0},loop=True)

        if bs.isTestBuild():
            self.betaInfo = bs.NodeActor(None,bs.Actor.UNRETAINED,
                                         'text',{'vAttach':'center',
                                                 'hAlign':'center',
                                                 'color':(1,1,1,1),
                                                 'scale':[1],
                                                 'translate':(230,35),
                                                 'text':bs.getResource('publicBetaText')},
                                         name='Public Beta')
            if not gDidInitialTransition:
                bsUtils.animate(self.betaInfo.node,'opacity',{1300:0,2000:1.0})


            if not gDidInitialTransition:
                self.betaInfo2 = bs.NodeActor(None,bs.Actor.UNRETAINED,
                                              'text',{'vAttach':'center',
                                                      'hAlign':'left',
                                                      'color':(1,1,1,1),
                                                      'scale':[0.6],
                                                      'translate':(160,10),
                                                      'text':('Thanks for helping beta-test!\n'
                                                              +('Note that Game-Center and iCloud syncing\n'
                                                                'do not function in this build.\n' if 'Mac' in bs.getUserAgentString() else '')
                                                              +'Please email any bugs or other feedback to\n'
                                                              'support@froemling.net or just post it on the\n'
                                                              'BombSquad facebook page.\n'
                                                              'Enjoy!\n'
                                                              '-eric')},
                                              name='Beta Info')
                if not gDidInitialTransition:
                    bsUtils.animate(self.betaInfo2.node,'opacity',{2300:0,3000:1.0,10000:1.0,11000:0.0})


        logoTexture = bs.getTexture("logo")
        logoModel = bs.getModel("logo")
        logoModelTransparent = bs.getModel("logoTransparent")
        model = bs.getModel("thePadLevel")
        treesModel = bs.getModel("trees")
        bottomModel = bs.getModel("thePadLevelBottom")
        testColorTexture = bs.getTexture("thePadLevelColor")
        treesTexture = bs.getTexture("treesColor")

        bgTex = bs.getTexture("menuBG")

        bgModel = bs.getModel("thePadBG")

        bs.setAttr(bs.globals,"mode","rotate")

        tint = (1.14,1.1,1.0)
        bs.setAttr(bs.globals,"tint",tint)
            
        bs.setAttr(bs.globals,"ambientColor",(1.06,1.04,1.03))
        #bs.setAttr(bs.globals,"vignetteOuter",(0.56,0.61,0.60))
        bs.setAttr(bs.globals,"vignetteOuter",(0.45,0.55,0.54))
        bs.setAttr(bs.globals,"vignetteInner",(0.99,0.98,0.98))

        self.terrain = bs.NodeActor(None,bs.Actor.UNRETAINED,
                                    "terrain",{'model':model,
                                               'colorTexture':testColorTexture,
                                               'reflection':'soft',
                                               'reflectionScale':[0.3]},
                                    name='Main Menu Model')
        
        self.bottom = bs.NodeActor(None,bs.Actor.UNRETAINED,
                                   "terrain",{'model':bottomModel,
                                              'lighting':False,
                                              'reflection':'soft',
                                              'reflectionScale':[0.45],
                                              'colorTexture':testColorTexture},
                                   name='Main Menu Model Bottom')

        self.trees = bs.NodeActor(None,bs.Actor.UNRETAINED,
                                  "terrain",{'model':treesModel,
                                             'lighting':False,
                                             'reflection':'char',
                                             'reflectionScale':[0.1],
                                             'colorTexture':treesTexture},
                                  name='Main Menu Trees')

        self.bg = bs.NodeActor(None,bs.Actor.UNRETAINED,
                               "terrain",{'model':bgModel,
                                          'color':(0.92,0.91,0.9),
                                          'lighting':False,
                                          'background':True,
                                          'colorTexture':bgTex},
                               name='Main Menu BG')

        textOffsetV = 0
        ts = 0.86

        # pop the logo and menu in
        def makeLogo(x,y,scale,delay):
            logo = bs.NodeActor(None,bs.Actor.UNRETAINED,
                                "image",{'texture':logoTexture,
                                         'modelOpaque':logoModel,
                                         'modelTransparent':logoModelTransparent,
                                         'attach':"center",
                                         'tiltTranslate':0.21,
                                         'absoluteScale':True},
                                name='Main Menu Logo')
            self._actors.append(logo)

            c = bs.newNode("combine",owner=logo.node,name='Logo Translate Driver')
            bs.connectAttr(c,'output',logo.node,'translate')

            keys = {}
            timeV = 0
            # gen some random keys for that stop-motion-y look
            for i in range(10):
                keys[timeV] = x+(random.random()-0.5)*0.7
                timeV += random.random() * 100
            bsUtils.animate(c,"input0",keys,loop=True)
            keys = {}
            timeV = 0
            for i in range(10):
                keys[timeV*ts] = y+(random.random()-0.5)*0.7
                timeV += random.random() * 100
            bsUtils.animate(c,"input1",keys,loop=True)

            c = bs.newNode("combine",owner=logo.node,attrs={"size":2},name='Logo Scale Driver')

            keys = {delay:0,delay+100:700*scale,delay+200:600*scale}
            bsUtils.animate(c,"input0",keys)
            bsUtils.animate(c,"input1",keys)
            bs.connectAttr(c,"output",logo.node,"scale")

        def makeWord(word,x,y,scale=1.0,delay=0):
            
            wordShadowObj = bs.NodeActor(None,bs.Actor.UNRETAINED,
                                         'text',{'translate':(x,y),
                                                 'big':True,
                                                 'color':(0.0,0.0,0.2,0.3),
                                                 'tiltTranslate':0.09,
                                                 'vAlign':'center',
                                                 'projectScale':0.97*scale,
                                                 'scale':[1.0],
                                                 'text':word},
                                         name='word shadow')

            self._actors.append(wordShadowObj)

            wordObj = bs.NodeActor(None,bs.Actor.UNRETAINED,
                                   'text',{'translate':(x,y),
                                           'big':True,
                                           'color':(1.2,1.15,1.15,1.0),
                                           'tiltTranslate':0.11,
                                           'vAlign':'center',
                                           'projectScale':scale,
                                           'scale':[1.0],
                                           'text':word},
                                   name='word')

            self._actors.append(wordObj)

            # shake 'bomb'
            c = bs.newNode("combine",owner=wordObj.node,name='Word Translate Driver')
            c2 = bs.newNode("combine",owner=wordShadowObj.node,name='Word Translate Driver')
            bs.connectAttr(c,'output',wordObj.node,'translate')
            bs.connectAttr(c2,'output',wordShadowObj.node,'translate')
            keys = {}
            keys2 = {}
            timeV = 0
            for i in range(10):
                val = x+(random.random()-0.5)*0.8
                val2 = x+(random.random()-0.5)*0.8
                keys[timeV*ts] = val
                keys2[timeV*ts] = val2+5
                timeV += random.random() * 100
            bsUtils.animate(c,"input0",keys,loop=True)
            bsUtils.animate(c2,"input0",keys2,loop=True)
            keys = {}
            keys2 = {}
            timeV = 0
            for i in range(10):
                val = y+(random.random()-0.5)*0.8
                val2 = y+(random.random()-0.5)*0.8
                keys[timeV*ts] = val
                keys2[timeV*ts] = val2-9
                timeV += random.random() * 100
            bsUtils.animate(c,"input1",keys,loop=True)
            bsUtils.animate(c2,"input1",keys2,loop=True)


            bsUtils.animate(wordObj.node,"projectScale",{delay:0.0, delay+100:scale*1.1, delay+200:scale})
            bsUtils.animate(wordShadowObj.node,"projectScale",{delay:0.0, delay+100:scale*1.1, delay+200:scale})

        # self._actors.append(bs.NodeActor(None,bs.Actor.UNRETAINED,
        #                                  'text',{'translate':(0,0),
        #                                          'big':True,
        #                                          'vAlign':'center',
        #                                          'projectScale':1.0,
        #                                          'scale':[1.0],
        #                                          'text':'TESTING'},
        #                                  name='word'))

        # self._actors.append(bs.NodeActor(None,bs.Actor.UNRETAINED,
        #                                  'text',{'translate':(0,-100),
        #                                          'big':True,
        #                                          'hAlign':'center',
        #                                          'vAlign':'center',
        #                                          'scale':[0.5],
        #                                          'text':'Z'},
        #                                  name='word'))
        # self._actors.append(bs.NodeActor(None,bs.Actor.UNRETAINED,
        #                                  'text',{'translate':(0,-100),
        #                                          'big':True,
        #                                          'hAlign':'center',
        #                                          'vAlign':'center',
        #                                          'scale':[0.2],
        #                                          'text':'Z'},
        #                                  name='word'))
        # self._actors.append(bs.NodeActor(None,bs.Actor.UNRETAINED,
        #                                  'text',{'translate':(0,-200),
        #                                          'big':True,
        #                                          'vAlign':'center',
        #                                          'projectScale':1.0,
        #                                          'scale':[0.5],
        #                                          'text':'TESTINGMORESTUFF'},
        #                                  name='word'))

        baseX = -170
        gScale = 1.1

        x = baseX-20
        y = 20
        spacing = 55*gScale

        baseDelay = 1000
        delay = baseDelay
        delayInc = 20
        
        # come on faster after the first time
        if gDidInitialTransition:
            baseDelay = 0
            delay = baseDelay
            delayInc = 20


        makeWord('B',x-50,y-23,scale=1.3*gScale,delay=delay)
        x += spacing; delay += delayInc
        # makeWord('o',x,y)
        # x += spacing
        makeWord('m',x,y,delay=delay,scale=gScale)
        x += spacing*1.25; delay += delayInc
        makeWord('b',x,y,delay=delay,scale=gScale)
        x += spacing*0.85; delay += delayInc

        makeWord('S',x,y-23,scale=1.3*gScale,delay=delay)
        x += spacing; delay += delayInc
        makeWord('q',x,y,delay=delay,scale=gScale)
        x += spacing*0.9; delay += delayInc
        makeWord('u',x,y,delay=delay,scale=gScale)
        x += spacing*0.9; delay += delayInc
        makeWord('a',x,y,delay=delay,scale=gScale)
        x += spacing*0.95; delay += delayInc
        makeWord('d',x,y,delay=delay,scale=gScale)

        makeLogo(baseX-28,125+y,0.32*gScale,delay=baseDelay)
            
        # a second or two into the main menu is a good time to preload some stuff we'll need elsewhere
        # to avoid hitches later on..
        def _preload1():

            bs.getModel('plasticEyesTransparent')
            bs.getModel('playerLineup1Transparent')
            bs.getModel('playerLineup2Transparent')
            bs.getModel('playerLineup3Transparent')
            bs.getModel('playerLineup4Transparent')
            bs.getModel('angryComputerTransparent')
            bs.getModel('scrollWidgetShort')
            bs.getModel('windowBGBlotch')

            bs.getTexture("playerLineup")
            bs.getTexture("lock")

            # preload stuff that gets used in co-op selection UI since that might be up soon
            for tex in ['iconRunaround',
                        'iconOnslaught',
                        'iconCampaign',
                        'medalComplete',
                        'medalBronze',
                        'medalSilver',
                        'medalGold']: bs.getTexture(tex)

            bs.getTexture("bg")


            bs.gameTimer(100,_preload2)

        def _preload2():

            # FIXME - could integrate these loads with the classes that use them
            # so they don't have to redundantly call the load (even if the actual result is cached)
            bs.getModel("powerup")
            bs.getModel("powerupSimple")
            bs.getTexture("powerupBomb")
            bs.getTexture("powerupSpeed")
            bs.getTexture("powerupPunch")
            bs.getTexture("powerupIceBombs")
            bs.getTexture("powerupStickyBombs")
            bs.getTexture("powerupShield")
            bs.getTexture("powerupImpactBombs")
            bs.getTexture("powerupHealth")
            bs.getSound("powerup01")
            bs.getSound("boxDrop")
            bs.getSound("boxingBell")
            bs.getSound("scoreHit01")
            bs.getSound("scoreHit02")
            bs.getSound("dripity")
            bs.getSound("spawn")
            bs.getSound("gong")

            bsBomb.getFactory()

            bs.gameTimer(100,_preload3)

        def _preload3():

            bs.getModel("bomb")
            bs.getModel("bombSticky")
            bs.getModel("impactBomb")
            bs.getTexture("bombColor")
            bs.getTexture("bombColorIce")
            bs.getTexture("bombStickyColor")
            bs.getTexture("impactBombColor")
            bs.getTexture("impactBombColorLit")
            bs.getSound("freeze")
            bs.getSound("fuse01")
            bs.getSound("activateBeep")
            bs.getSound("warnBeep")

            spazFactory = bsSpaz.getFactory()

            # go through and load our existing spazzes
            # (spread these out quite a bit)
            def _load(spaz):
                spazFactory.preload(spaz)
                # icons also..
                bs.getTexture(bsSpaz.appearances[spaz].iconTexture,minQuality='High')
                bs.getTexture(bsSpaz.appearances[spaz].iconMaskTexture)

            t = 50
            for spaz in bsSpaz.appearances:
                bs.gameTimer(t,bs.Call(_load,spaz))
                t += 100

            bs.gameTimer(200,_preload4)


        def _preload4():

            bsUtils.playMusic('Menu')

            bs.getTexture("frame")
            bs.getTexture("frameInset")
            bs.getModel('frameInset')
            bs.getSound("metalHit")
            bs.getSound("metalSkid")
            bs.getSound("refWhistle")
            bs.getTexture("bar")
            bs.getTexture("meter")
            bs.getTexture("null")
            bs.getModel("meterTransparent")
            bs.getTexture("flagColor")
            bs.getSound("achievement")
            bs.getTexture("achievementOutline")
            bs.getModel("achievementOutline")
            bsPowerup.getFactory()

        # hopefully this won't hitch but lets space these out anyway..
        bs.addCleanFrameCallback(_preload1)

        random.seed()

        # on the main menu, also show our news..
        class News(object):
            def __init__(self):
                self._valid = True

                self._messageDuration = 10000
                self._messageSpacing = 2000
                self._text = None

                # if we've got a valid cached one, use that immediately; otherwise go out to the server
                global gNews
                try: self._gotNews({'news':gNews})
                except Exception: self._fetchNews()

            def _fetchNews(self):
                try: launchCount = bs.config['launchCount']
                except Exception: launchCount = None
                global gLastNewsFetchTime
                gLastNewsFetchTime = time.time()
                bsUtils.serverGet('bsNews',{'v':'2','lc':launchCount,'t':int(gLastNewsFetchTime-gStartTime)},bs.WeakCall(self._gotNews))

            def _changePhrase(self):

                global gLastNewsFetchTime
                # if our news is way out of date, lets re-request it.. otherwise, rotate our phrase
                if time.time()-gLastNewsFetchTime > 600.0:
                    self._fetchNews()
                    self._text = None
                else:
                    if self._text != None:
                        if len(self._phrases) == 0:
                            for p in self._usedPhrases:
                                self._phrases.insert(0,p)
                        s = self._messageSpacing
                        keys = {s:0,s+1000:1.0,s+self._messageDuration-1000:1.0,s+self._messageDuration:0.0}
                        bsUtils.animate(self._combine,"input3",dict([[k,v*0.5] for k,v in keys.items()]))
                        bs.setAttr(self._text.node,'text',self._phrases.pop())

            def _gotNews(self,data):
                if data == None: return
                news = str(data['news'])

                # throw out a subtle notice if a new version is available
                try:
                    if 'latestVersion' in data:
                        # strip off any 'b1' type stuff
                        rawVersion = bs.getVersion().split('b')[0]
                        ourVersion = [int(n) for n in rawVersion.split('.')]
                        while len(ourVersion) < 3: ourVersion.append(0)
                        newestVersion = [int(n) for n in data['latestVersion'].split('.')]
                        while len(newestVersion) < 3: newestVersion.append(0)
                        if newestVersion > ourVersion:
                            def foo():
                                bs.screenMessage("A newer version of BombSquad is available!  ("+data['latestVersion']+')',(0.7,1,0))
                                bs.playSound(bs.getSound('ding'))
                            bs.gameTimer(2000,foo)
                except Exception,e:
                    print 'Exception comparing versions:',e

                # cache this..
                global gNews
                gNews = news

                # our news data now starts with 'BSNews:' so we can filter out
                # result that arent actually coming in from our server
                # (such as wireless access point setup pages)
                if not news.startswith("BSNews:"): return
                news = news[7:]
                if news == '' or not self._valid: return
                self._phrases = []
                self._usedPhrases = [s for s in news.split('<br>\r\n')]
                self._phraseChangeTimer = bs.Timer(self._messageDuration+self._messageSpacing,bs.WeakCall(self._changePhrase),repeat=True)

                self._text = bs.NodeActor(None,bs.Actor.UNRETAINED,
                                          'text',{'vAttach':'top',
                                                  'hAttach':'left',
                                                  'hAlign':'left',
                                                  'vAlign':'top',
                                                  'color':(0.7,0.8,1.0,0.5),
                                                  'scale':[0.9] if bs.getInterfaceType() == 'phone' else [0.6],
                                                  'translate':(10,-10),
                                                  'text':news[0]},
                                          name='News Text')

                self._combine = bs.newNode("combine",owner=self._text.node,attrs={'input0':1.0,'input1':0.8,'input2':1.0},name='News Color Driver')
                bs.connectAttr(self._combine,'output',self._text.node,'color')
                self._changePhrase()

        self._news = News()

        # bring up the last place we were at, or start at the main menu otherwise
        try: mainWindow = bsUI.gMainWindow
        except Exception: mainWindow = None

        if mainWindow == 'Team Game Select': bsUI.uiGlobals['mainMenuWindow'] = bsUI.PlaylistWindow(transition=None).getRootWidget()
        elif mainWindow == 'Free-for-All Game Select': bsUI.uiGlobals['mainMenuWindow'] = bsUI.PlaylistWindow(transition=None,playlistType='freeForAll').getRootWidget()
        elif mainWindow == 'Coop Select': bsUI.uiGlobals['mainMenuWindow'] = bsUI.CooperativeSelectWindow(transition=None).getRootWidget()
        else: bsUI.uiGlobals['mainMenuWindow'] = bsUI.MainMenuWindow(transition=None).getRootWidget()

        gDidInitialTransition = True

    def onPlayerRequest(self,playerID):
        return False

    def getSessionType(self):
        return 'mainMenu'
